#!/usr/bin/perl
use strict;
use warnings;
use RFC::Main;
use RFC::Rig;
use RFC::Cache;
my $prog = $0;
sleep(15);
my $r = RFC::Rig->new(shift);
$r->check_run();
my @gasses = $r->get_cv('main','dual_monitor_gasses');
my $data = RFC::Cache->new();
my $debug = 0;
my @notice_emails = $r->get_cv('main','notice_mails');
$debug = 1 if (@ARGV);
foreach my $gas (@gasses)
{
    my $dev_p1 = $r->get_device('filter',$gas.'_north');
    $dev_p1 = $r->get_device('filter',$gas.'_west') unless ($dev_p1);
    my $dev1_time = $r->get_device('math',$gas.'_north_remaining_time');
    $dev1_time = $r->get_device('math',$gas.'_west_remaining_time') unless ($dev1_time);
    my $dev_p2 = $r->get_device('filter',$gas.'_south');
    $dev_p2 = $r->get_device('filter',$gas.'_east') unless ($dev_p2);
    my $dev2_time = $r->get_device('math',$gas.'_south_remaining_time');
    $dev2_time = $r->get_device('math',$gas.'_east_remaining_time') unless ($dev2_time);
    if ($debug)
    {
	print $dev_p1,"\n";
	print $dev_p2,"\n";
	print $dev1_time,"\n";
	print $dev2_time,"\n";
    }
    next unless ($dev_p1 && $dev_p2 && $dev1_time && $dev2_time);
    my $p1 = $dev_p1->read($data);
    my $p2 = $dev_p2->read($data);
    if ($p1 < 0 || $p2 < 0)
    {
	$data = RFC::Cache->new();
	$p1 = $dev_p1->read($data);
	$p2 = $dev_p2->read($data);
    }
    my $t1 = $dev1_time->read($data); 
    my $t2 = $dev2_time->read($data); 
    my $gas1 = {'nr' => 1,'p' => $p1,'t' => $t1,'pdev' => $dev_p1,'tdev'=>$dev1_time,'name'=>$dev_p1->name};
    my $gas2 = {'nr' => 2,'p' => $p2,'t' => $t2,'pdev' => $dev_p2,'tdev'=>$dev2_time,'name'=>$dev_p2->name};
    foreach ($gas1,$gas2)
    {
	my $nr = $$_{'nr'};
	if ($debug)
	{
	    print "$gas $nr ".$$_{'name'}.":\n";
	    print "$gas $nr pressure: ",$$_{'p'},"\n";
	    print "$gas $nr time: ",$$_{'t'},"\n";
	}
	$$_{'t'} = 1000 if (defined($$_{'t'}) && $$_{'t'} < 0.01);  ## a rate of 0 will give 0 as time... ##
	$$_{'odev'} = $r->get_device('relay',$$_{'pdev'}->get_conf('output'));
	print "$gas $nr corrected time: ",$$_{'t'},"\n" if ($debug);
    }
    my $p_green_limit = $r->get_cv($gas.'_levels','green_pressure');
    my $p_yellow_limit = $r->get_cv($gas.'_levels','yellow_pressure');
    my $time_limit = $r->get_cv($gas.'_levels','time_limit');
    my $hysteresis = $r->get_cv($gas.'_levels','hysteresis');
    $hysteresis = 1 unless ($hysteresis && $hysteresis > 0);
    $p_green_limit = 15 unless ($p_green_limit);  
    $p_yellow_limit = 25 unless ($p_yellow_limit);
    $time_limit = 12 unless ($time_limit);
    my $green = 0;
    my $yellow = 0;
    my $red = 0;
    my $time = 0;
    my $remaining = 0;
    my $reset = 0;
    my @l = sort {$$a{'p'} <=> $$b{'p'}} ($gas1,$gas2);
    foreach my $g (@l)
    {
	print $$g{'name'},"\n" if ($debug);
	print $$g{'p'},"\n" if ($debug);
    }
    if ($l[0]{'p'} < $p_green_limit)
    {
	$green = 1;
	$yellow = 1 if ($l[1]{'p'} < $p_yellow_limit);
	if ($l[1]{'t'} < $time_limit)
	{
	    $red = 1;
	    $time = 1;
	    $remaining = $l[1]{'t'};
	}
	if ($l[1]{'p'} < $p_green_limit)
	{
	    $red = 1;
	    $time = 0;
	}
    }
    elsif ($l[0]{'p'} > $p_green_limit + $hysteresis)
    {
	$reset = 1;
    }
    my $d1 = $$gas1{'odev'};
    my $d2 = $$gas2{'odev'};
    if ($red)
    {
	print "Alert: $gas has run out!\n" if ($debug);
	$d1->set_noinfo(0,$prog,'nolog');
	$d2->set_noinfo(1,$prog,'nolog');
	if ($time)
	{
	    my $str = <<END;

Alert!

$gas is about to run out!
With the current gas usage, less than $time_limit hours remain!

estimatedtime remaining: $remaining hours!

END

            $r->warning_mail($gas.'_red',$str,'Alert: '.$gas.' is about to run out!');
	}
	else
	{
	    my $str = <<END;

Alert!

$gas has run out!

Measured pressures:
$p1
$p2

END


            $r->warning_mail($gas.'_red',$str,'Alert: '.$gas.' has run out!');
	}
    }
    elsif ($yellow)
    {
	print "Warning: $gas is running out!\n" if ($debug);
	$d1->set_noinfo(1,$prog,'nolog');
	$d2->set_noinfo(1,$prog,'nolog');
	my $str = <<END;

Alert!

$gas is about to run out!
There is less than $p_yellow_limit bar left in both stations!

Measured pressures:
$p1
$p2

END


        $r->warning_mail($gas.'_yellow',$str,'Alert: '.$gas.' is about to run out!');	    
    }
    elsif ($green)
    {
	print "$gas needs change\n" if ($debug);
	$d1->set_noinfo(1,$prog,'nolog');
	$d2->set_noinfo(0,$prog,'nolog');
	my $status = $r->check_warning($gas.'_green');
	if (defined($status) && !$status)
	{
	    foreach my $email (@notice_emails)
	    {
		RFC::Main::mail($email,$gas." needs to be changed as gas pressure is below $p_green_limit for one of the stations",'Warning: '.$gas." is below $p_green_limit");
	    }
	    $r->set_warning($gas.'_green');
	}
    }
    elsif ($reset)
    {
	print "All ok for $gas\n" if ($debug);
	$d1->set_noinfo(0,$prog,'nolog');
	$d2->set_noinfo(0,$prog,'nolog');
        $r->unset_warning($gas.'_red');
        $r->unset_warning($gas.'_yellow');
        $r->unset_warning($gas.'_green');
    }
    if ($debug)
    {
	print $d1,' : ',$d1->read(),"\n";
	print $d2,' : ',$d2->read(),"\n";
    }
}
