#!/usr/bin/perl -T
use strict;
use warnings;
use CGI qw(:standard :netscape);
use RFC::Main;
use RFC::Rig;
use RFC::RFCCGI qw(:cgi :html);
use RFC::Header;
use RFC::Cache;
use Fcntl qw(:DEFAULT :flock);

unless (param)
{
    print header(-access_control_allow_origin => '*',-content_type=>'text/plain',-charset=>'utf-8');
    print <<END;


Oops.. no parameters, displaying interface definitions!

Common parameters:

Parameter 'action', determines which function to call (each described below)
Generally, the script returns OK on the first line if input is valid, and an errorstring if not.
Thus remember to check for the 'OK' string as part of the ajax response function.


check_status
Returns the status of the requested tag. All tags must be listed in the 'gasmon' secton in the rig configuration file
Only tags found in one fo the following lists can be checked:
dual_gas_list
single_gas_list
remote_gas_list

Parameters:
'rig'  Rig number, integer
'device' tagname ,string


get_values
Returns a list of values for the specified simplechannel devices.
The response will be in the form:

OK
devname1:value1
devname2:value2
....


Parameters:
'rig'  Rig number, integer
'devices' tagnames , comma separated string (devname1,devname2,...)


END
exit;
}

print header();
my $ac = param('action');
unless ($ac)
{
    print "";
    exit;
}


my $r = param('rig');
unless ($r)
{
    print header();
    print "No rig specified!\n";
    exit;
}
my $rig = RFC::Rig->new($r);

if ($ac eq 'get_values')
{
    ### Aquire lock to make suer no other data logging is running ###
    my $lockfile = '/tmp/rig_'.$rig->name.'_logfile.lock';
    unless (-e $lockfile)
    {
	sysopen(FH, $lockfile, O_WRONLY | O_CREAT) or CORE::die "Error: Could not open $lockfile $!";
	chmod 0666, $lockfile;
    }
    else
    {
	sysopen(FH, $lockfile, O_WRONLY | O_CREAT) or CORE::die "Error: Could not open $lockfile $!";
    }
    flock(FH,LOCK_EX);
    ### Only use lock to wait for logfile.pl, do not delay unlocking as lockfile.pl may be prevented from starting!
    flock(FH,LOCK_UN);
    close(FH);
    my $str = param('devices');
    my @devs = ();
    $str = '' unless ($str);
    foreach my $n (split(/\s*\,\s*/,$str))
    {
	next unless ($n);
	my $d = $rig->init_device('simplechannel',$n);
	push @devs,$d if ($d);
    }
    unless (@devs)
    {
	print "Error: no devices specified or no simplechannel devices matched specified names!";
	exit;
    }
    print "OK\n";
    my $data = RFC::Cache->new();
    foreach my $dev (@devs)
    {
	print $dev->name,':',$dev->read_cache($data),"\n";
    }
    exit;
}

if ($ac eq 'check_status')
{
    my $id = param('device');
    my @list = $rig->get_cv('gasmon','dual_gas_list');
    foreach my $name (@list)
    {
	next unless ($name eq $id);
	print "OK\n";
	print "$id\n";
	my $type = $rig->get_cv('gasmon',$id.'_type');
	my @devs = $rig->get_cv('gasmon','devices_'.$id);
	my $status = 'ok';
	foreach my $dn (@devs)
	{
	    my $d = $rig->get_device($type,$dn);
	    next unless ($d);
	    my $str = $dn;
	    if ($d->read())
	    {
		$str =~ /_(.*)/;
		$status = $1;
		last;
	    }
	}
	print $status,"\n";
	exit;
    }
    @list = $rig->get_cv('gasmon','single_gas_list');
    foreach my $name (@list)
    {
	next unless ($name eq $id);
	print "OK\n";
	print "$id\n";
	my $status = 'green';	
	my $type = $rig->get_cv('gasmon',$id.'_type');
	my $dev = $rig->get_cv('gasmon',$id.'_device');
	my $d = $rig->get_device($type,$dev);
	$status = 'red' if ($d && $d->read());
	print $status,"\n";
	exit;
    }
    @list = $rig->get_cv('gasmon','remote_gas_list');
    foreach my $name (@list)
    {
	next unless ($name eq $id);
	print "OK\n";
	print "$id\n";
	my $status = 'green';
	my $type = $rig->get_cv('gasmon',$id.'_type');
	my $dev = $rig->get_cv('gasmon',$id.'_device');
	my $d = $rig->get_device($type,$dev);
	$status = 'red' if ($d && $d->read());
	print $status,"\n";
	exit;
    }
    print "Device $id not found in list of avaliable monitoiring devices!\n";
    exit;
}
